package za.co.momentum.recurring.nurse.client.sao.suburb;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Platform;
import za.co.momentum.recurring.nurse.client.model.SuburbUploader;
import za.co.momentum.recurring.nurse.client.sao.AbstractServiceManager;
import za.co.momentum.risk.fastlane.nurse.dto.SuburbTO;
import za.co.momentum.recurring.nurse.util.ClientProperties;

/**
 * The suburb service manager will manage the suburbs availible in the system.
 * Currently the client will receive a file containing the latest suburb info
 * and then save it to disk. This singleton class will then read that file and
 * create the suburb list.
 * 
 * @author NideBruyn Created on 10 Nov, 2005 Risk & Savings
 * 
 */
public class SuburbServiceManager extends AbstractServiceManager {
	/*
	 * A single instance of this class. It uses the singleton pattern.
	 */
	private static SuburbServiceManager instance;
	/*
	 * This is the LOG instance.
	 */
	private static final Logger LOG = Logger
			.getLogger(SuburbServiceManager.class);
	/*
	 * A property holding the list of suburbs.
	 */
	private List suburbs;

	/**
	 * This is the hidden constructor so that we can make the class a singleton...
	 * 
	 */
	protected SuburbServiceManager() {
	}

	/**
	 * Method for returning an instance of this class.
	 * 
	 * @return
	 */
	public static SuburbServiceManager getInstance() {
		if (instance == null) {
			synchronized (SuburbServiceManager.class) {
				if (instance == null) {
					instance = new SuburbServiceManager();
				}
			}
		}
		return instance;
	}

	/**
	 * This class will read all the suburbs from the file system from a suburb
	 * file. Look in the system.properties file for the property: -
	 * suburbFile=c:/Base/20051002.txt
	 * 
	 * @return
	 */
	public List getSuburbListOld() {
		if (suburbs == null) {
			SuburbUploader su = new SuburbUploader(ClientProperties.getInstance()
					.getProperty(ClientProperties.CLIENT_RESOURCES));
			suburbs = su.getSuburbs();
			// Sort suburbs
			Map m = new TreeMap();
			for (Iterator i = suburbs.iterator(); i.hasNext();) {
				SuburbTO s = (SuburbTO) i.next();
				m.put(s.getSuburbName(), s);
			}
			suburbs.clear();
			for (Iterator i = m.keySet().iterator(); i.hasNext();) {
				suburbs.add(m.get(i.next()));
			}
			LOG.debug("Loaded suburbs");
		}
		return suburbs;
	}

	/**
	 * This class will read all the suburbs from the file system from a suburb
	 * file. Look in the system.properties file for the property: -
	 * file.suburb=suburb.txt
	 * 
	 * @return
	 */
	public List getSuburbList() {
		if (suburbs == null) {
			String path = Platform.getInstallLocation().getURL().getPath();
			if (path.charAt(0) == '/') {
				path = path.substring(1, path.length());
			}
			LOG.info("New suburb method is now going to be done");
	/*		try {
				LOG.debug("Bundle is " + Platform.getBundle("TNResources"));
				LOG.debug("Local URL is "
						+ Platform.asLocalURL(Platform.getBundle("TNResources").getEntry(
								"/resource/"
										+ ClientProperties.getInstance().getProperty(
												ClientProperties.FILE_SUBURB))));
				LOG.debug("Suburb file is:"
						+ Platform.asLocalURL(
								Platform.getBundle("TNResources").getEntry(
										"/resource/"
												+ ClientProperties.getInstance().getProperty(
														ClientProperties.FILE_SUBURB))).getPath());
			} catch (IOException e) {
				LOG.error("Error reading suburb file" + e.getMessage());
			}*/
			File file = null;
			try {
				file = new File(Platform.asLocalURL(
						Platform.getBundle("TNResources").getEntry(
								"/resource/"
										+ ClientProperties.getInstance().getProperty(
												ClientProperties.FILE_SUBURB))).getPath());
			} catch (IOException e) {
				LOG.error("Error reading suburb file " + e.getMessage());
			}
			if (file != null && file.exists()) {
				suburbs = parseFile(file);
				// Sort suburbs
				Map m = new TreeMap();
				for (Iterator i = suburbs.iterator(); i.hasNext();) {
					SuburbTO s = (SuburbTO) i.next();
					m.put(s.getSuburbName(), s);
				}
				suburbs.clear();
				for (Iterator i = m.keySet().iterator(); i.hasNext();) {
					suburbs.add(m.get(i.next()));
				}
				LOG.debug("Loaded suburbs");
			}
		}
		return suburbs;
	}

	private List parseFile(File file) {
		List list = new ArrayList();
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line = br.readLine();
			do {
				line = br.readLine();
				if (line != null) {
					StringTokenizer st = new StringTokenizer(line, ",");
					try {
						SuburbTO s = new SuburbTO();
						s.setId(new Long(st.nextElement().toString().trim()));
						s.setCity(st.nextElement().toString().trim());
						s.setCode(st.nextElement().toString().trim());
						s.setSuburbName(st.nextElement().toString().trim());
						list.add(s);
					} catch (Exception ee) {
						LOG.warn("Format error in file " + line + " " + ee.getMessage());
					}
				}
			} while (line != null);
			br.close();
		} catch (FileNotFoundException e) {
			LOG.error("File not found " + file + " " + e.getMessage());
			return null;
		} catch (IOException e) {
			LOG.error("IOException " + e.getMessage());
			return null;
		}
		return list;
	}

	/*
	 * Will return first id of suburb that matches string If more than one suburb
	 * name is the same ?
	 */
	public SuburbTO getSuburb(String suburbName) {
		for (Iterator iter = suburbs.iterator(); iter.hasNext();) {
			SuburbTO element = (SuburbTO) iter.next();
			if (suburbName.equalsIgnoreCase(element.getSuburbName())) {
				return element;
			}
		}
		return null;
	}
}
